@@ -35,6 +35,25 @@ _absent = object()
 # values, but must be encoded in URL Query keys and values.
 #
 
+# Map of various URL schemes to their default ports. Scheme strings are
+# lowercase.
+DEFAULT_PORTS = {
+    'ftp': 21,
+    'ssh': 22,
+    'http': 80,
+    'https': 443,
+    }
+
+# List of schemes that don't require two slashes after the colon. For example,
+# 'mailto:user@google.com' instead of 'mailto://user@google.com'. Scheme
+# strings are lowercase.
+#
+# TODO(grun): Support schemes separated by just ':', not '://' without having an
+# explicit list. There are many such schemes in various URIs.
+COLON_SEPARATED_SCHEMES = [
+    'mailto',
+    ]
+
 
 class Path(object):
 
@@ -447,11 +466,23 @@ class Query(object):
     def remove(self, query):
         if query is True:
             self.load('')
-        else:
-            keys = [query]
-            if callable_attr(query, '__iter__'):
-                keys = query
-            for key in keys:
+            return self
+
+        # Single key to remove.
+        items = [query]
+        # Dictionary or multivalue dictionary of items to remove.
+        if callable_attr(query, 'iteritems'):
+            items = self._items(query)
+        # List of keys or items to remove.
+        elif callable_attr(query, '__iter__'):
+            items = query
+
+        for item in items:
+            if callable_attr(item, '__iter__') and len(item) == 2:
+                key, value = item
+                self.params.popvalue(key, value, None)
+            else:
+                key = item
                 self.params.pop(key, None)
         return self
 
@@ -547,7 +578,7 @@ class Query(object):
 
     def _extract_items_from_querystr(self, querystr):
         pairstrs = [s2 for s1 in querystr.split('&') for s2 in s1.split(';')]
-        
+
         if self.strict:
             pairs = map(lambda item: item.split('=', 1), pairstrs)
             pairs = map(lambda p: (p[0], '') if len(p) == 1
@@ -564,7 +595,8 @@ class Query(object):
         items = []
         parsed_items = urlparse.parse_qsl(querystr, keep_blank_values=True)
         for (key, value), pairstr in izip(parsed_items, pairstrs):
-            if key == urllib.quote_plus(pairstr):  # Empty value without '=', like '?sup'.
+            # Empty value without '=', like '?sup'.
+            if key == urllib.quote_plus(pairstr):
                 value = None
             items.append((key, value))
         return items
@@ -764,8 +796,7 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
       scheme://username:password@host:port/path?query#fragment
 
     Attributes:
-      DEFAULT_PORTS: Map of various URL schemes to their default
-        ports. Scheme strings are lowercase.
+      DEFAULT_PORTS: 
       strict: Boolean whether or not UserWarnings should be raised if
         improperly encoded path, query, or fragment strings are provided
         to methods that take such strings, like load(), add(), set(),
@@ -786,13 +817,6 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
       fragment: Fragment object from FragmentCompositionInterface.
     """
 
-    DEFAULT_PORTS = {
-        'ftp': 21,
-        'ssh': 22,
-        'http': 80,
-        'https': 443,
-    }
-
     def __init__(self, url='', strict=False):
         """
         Raises: ValueError on invalid url.
@@ -802,7 +826,7 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
         FragmentCompositionInterface.__init__(self, strict=strict)
         self.strict = strict
 
-        self.load(str(url))  # Raises ValueError on invalid url.
+        self.load(url)  # Raises ValueError on invalid url.
 
     def load(self, url):
         """
@@ -815,6 +839,10 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
         self._port = None
         self._scheme = None
 
+        if isinstance(url, unicode):
+            url = url.encode('utf-8')
+        url = str(url)
+
         # urlsplit() raises a ValueError on malformed IPv6 addresses in
         # Python 2.7+. In Python <= 2.6, urlsplit() doesn't raise a
         # ValueError on malformed IPv6 addresses.
@@ -823,7 +851,7 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
         self.netloc = tokens.netloc  # Raises ValueError in Python 2.7+.
         self.scheme = tokens.scheme
         if not self.port:
-            self._port = self.DEFAULT_PORTS.get(self.scheme)
+            self._port = DEFAULT_PORTS.get(self.scheme)
         self.path.load(tokens.path)
         self.query.load(tokens.query)
         self.fragment.load(tokens.fragment)
@@ -858,15 +886,14 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
     @port.setter
     def port(self, port):
         """
-        A port value can 1-65535 or None meaning no port specified. If
-        <port> is None and self.scheme is a known scheme in
-        self.DEFAULT_PORTS, the default port value from
-        self.DEFAULT_PORTS will be used.
+        A port value can 1-65535 or None meaning no port specified. If <port>
+        is None and self.scheme is a known scheme in DEFAULT_PORTS, the default
+        port value from DEFAULT_PORTS will be used.
 
         Raises: ValueError on invalid port.
         """
         if port is None:
-            self._port = self.DEFAULT_PORTS.get(self.scheme)
+            self._port = DEFAULT_PORTS.get(self.scheme)
         elif is_valid_port(port):
             self._port = int(str(port))
         else:
@@ -881,7 +908,7 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
             userpass += '@'
 
         netloc = self.host or ''
-        if self.port and self.port != self.DEFAULT_PORTS.get(self.scheme):
+        if self.port and self.port != DEFAULT_PORTS.get(self.scheme):
             netloc += ':' + str(self.port)
 
         netloc = ((userpass or '') + (netloc or ''))
@@ -1108,10 +1135,10 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
             self.fragment.separator = fragment_separator
         return self
 
-    def remove(
-        self, args=_absent, path=_absent, fragment=_absent, query=_absent,
-        query_params=_absent, port=False, fragment_path=_absent,
-            fragment_args=_absent, username=False, password=False):
+    def remove(self, args=_absent, path=_absent, fragment=_absent,
+               query=_absent, query_params=_absent, port=False,
+               fragment_path=_absent, fragment_args=_absent, username=False,
+               password=False):
         """
         Remove components of this furl's URL and return this furl
         instance, <self>.
@@ -1188,7 +1215,10 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
                 url = url[2:]
             elif url.startswith('://'):
                 url = url[3:]
-        elif (self.scheme is not None and 
+        elif self.scheme in COLON_SEPARATED_SCHEMES:
+            # Change a '://' separator to ':'. Leave a ':' separator as-is.
+            url = _set_scheme(url, self.scheme)
+        elif (self.scheme is not None and
               (url == '' or  # Protocol relative URL.
                (url == '%s:' % self.scheme and not str(self.path)))):
             url += '//'
@@ -1198,56 +1228,35 @@ class furl(URLPathCompositionInterface, QueryCompositionInterface,
         return "%s('%s')" % (self.__class__.__name__, str(self))
 
 
-"""
-urlparse.urlsplit() and urlparse.urljoin() don't separate the query
-string from the path for schemes not in the list urlparse.uses_query.
-furl will support proper parsing of query strings and paths for all
-schemes.
-
-As a workaround, use 'http' (a scheme in urlparse.uses_query) for the
-purposes of urlparse.urlsplit() and urlparse.urljoin(), but then revert
-back to the original scheme provided once urlsplit() or urljoin() has
-completed.
-
-_get_scheme() and _change_scheme() are helper methods for getting and
-setting the scheme of URL strings. Used to change the scheme to 'http'
-and back again.
-"""
-
 def _get_scheme(url):
-    if url.lstrip().startswith('//'):
-        return ''  # Protocol relative URL.
-    return url[:max(0,url.find('://'))] or None
+    scheme = None
+    if url.lstrip().startswith('//'):  # Protocol relative URL.
+        return ''
+    beforeColon = url[:max(0, url.find(':'))]
+    if beforeColon in COLON_SEPARATED_SCHEMES:
+        return beforeColon
+    return url[:max(0, url.find('://'))] or None
 
 
 def _set_scheme(url, newscheme):
-    newscheme = newscheme or ''
     scheme = _get_scheme(url)
+    newscheme = newscheme or ''
+    separator = ':' if scheme in COLON_SEPARATED_SCHEMES else '://'
+    newseparator = ':' if newscheme in COLON_SEPARATED_SCHEMES else '://'
     if scheme == '':  # Protocol relative URL.
         url = '%s:%s' % (newscheme, url)
     elif scheme is None and url:  # No scheme.
-        url = '%s://%s' % (newscheme, url)
+        url = ''.join([newscheme, newseparator, url])
     elif scheme:  # Existing scheme.
-        url = newscheme + url[len(scheme):]
+        remainder = url[len(scheme):]
+        if remainder.startswith('://'):
+            remainder = remainder[3:]
+        elif remainder.startswith(':'):
+            remainder = remainder[1:]
+        url = ''.join([newscheme, newseparator, remainder])
     return url
 
 
-def urljoin(base, url):
-    """
-    Parameters:
-      base: Base URL to join with <url>.
-      url: Relative or absolute URL to join with <base>.
-
-    Returns: The resultant URL from joining <base> and <url>.
-    """
-    base_scheme, url_scheme = _get_scheme(base), _get_scheme(url)
-    httpbase = _set_scheme(base, 'http')
-    joined = urlparse.urljoin(httpbase, url)
-    if not url_scheme:
-        joined = _set_scheme(joined, base_scheme)
-    return joined
-
-
 def urlsplit(url):
     """
     Parameters:
@@ -1267,14 +1276,31 @@ def urlsplit(url):
         l[0] = scheme
         return tuple(l)
 
-    # If a scheme wasn't provided, we shouldn't add one by setting the
-    # scheme to 'http'. We can use urlparse.urlsplit(url) as-is.
+    # urlsplit() only parses the query for schemes in urlparse.uses_query,
+    # so switch to 'http' (a scheme in urlparse.uses_query) for
+    # urlparse.urlsplit() and switch back afterwards.
     if original_scheme is not None:
         url = _set_scheme(url, 'http')
     toks = urlparse.urlsplit(url)
     return urlparse.SplitResult(*_change_urltoks_scheme(toks, original_scheme))
 
 
+def urljoin(base, url):
+    """
+    Parameters:
+      base: Base URL to join with <url>.
+      url: Relative or absolute URL to join with <base>.
+
+    Returns: The resultant URL from joining <base> and <url>.
+    """
+    base_scheme, url_scheme = urlsplit(base).scheme, urlsplit(url).scheme
+    httpbase = _set_scheme(base, 'http')
+    joined = urlparse.urljoin(httpbase, url)
+    if not url_scheme:
+        joined = _set_scheme(joined, base_scheme)
+    return joined
+
+
 def join_path_segments(*args):
     """
     Join multiple lists of path segments together, intelligently
