@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 #
 # furl - URL manipulation made simple.
 #
@@ -620,6 +621,27 @@ class TestQuery(unittest.TestCase):
             assert q.remove(True) == q
             assert len(q.params) == 0
 
+        # List of keys to remove.
+        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q.remove(['a', 'b'])
+        assert not q.params.items()
+
+        # List of items to remove.
+        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q.remove([('a', '1'), ('b', '3')])
+        assert q.params.allitems() == [('b', '2'), ('a', '4')]
+
+        # Dictionary of items to remove.
+        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        q.remove({'b':'3', 'a':'1'})
+        assert q.params.allitems() == [('b', '2'), ('a', '4')]
+
+        # Multivalue dictionary of items to remove.
+        q = furl.Query([('a','1'), ('b', '2'), ('b', '3'), ('a', '4')])
+        omd = omdict1D([('a', '4'), ('b', '3'), ('b', '2')])
+        q.remove(omd)
+        assert q.params.allitems() == [('a', '1')]
+
     def test_params(self):
         # Basics.
         q = furl.Query('a=a&b=b')
@@ -918,23 +940,29 @@ class TestFurl(unittest.TestCase):
         items = urlparse.parse_qsl(urlparse.urlsplit(url).query, True)
         return (key, val) in items
 
+
+    def test_unicode(self):
+        url = u'http://ru.wikipedia.org/wiki/Восход_(ракета-носитель)'
+        f = furl.furl(url)  # Accept a unicode URL without raising an exception.
+        assert not isinstance(f.url, unicode)  # URLs cannot contain unicode.
+
     def test_scheme(self):
         assert furl.furl().scheme is None
         assert furl.furl('').scheme is None
-
+        
         # Lowercase.
         assert furl.furl('/sup/').set(scheme='PrOtO').scheme == 'proto'
-
+        
         # No scheme.
         for url in ['sup.txt', '/d/sup', '#flarg']:
             f = furl.furl(url)
             assert f.scheme is None and f.url == url
-
+        
         # Protocol relative URLs.
         for url in ['//', '//sup.txt', '//arc.io/d/sup']:
             f = furl.furl(url)
             assert f.scheme == '' and f.url == url
-
+        
         f = furl.furl('//sup.txt')
         assert f.scheme == ''
         f.scheme = None
@@ -942,6 +970,20 @@ class TestFurl(unittest.TestCase):
         f.scheme = ''
         assert f.scheme == '' and f.url == '//sup.txt'
 
+        # Schemes without slashes , like 'mailto:'.
+        f = furl.furl('mailto:sup@sprp.ru')
+        assert f.url == 'mailto:sup@sprp.ru'
+        f = furl.furl('mailto://sup@sprp.ru')
+        assert f.url == 'mailto:sup@sprp.ru'
+
+        f = furl.furl('mailto:sproop:spraps@sprp.ru')
+        assert f.scheme == 'mailto'
+        assert f.username == 'sproop' and f.password == 'spraps'
+        assert f.host == 'sprp.ru'
+
+        f = furl.furl('mailto:')
+        assert f.url == 'mailto:' and f.scheme == 'mailto'
+
     def test_username_and_password(self):
         # Empty usernames and passwords.
         for url in ['', 'http://www.pumps.com/']:
